@@ -23,8 +23,8 @@
 #include <stdlib.h>
 #include <string.h>
 
-// GLib
-#include <glib.h>
+#include "zix/hash.h"
+#include "zix/tree.h"
 
 #include "sord-config.h"
 #include "sord_internal.h"
@@ -54,7 +54,7 @@
 #define DEFAULT_GRAPH_ORDER GSPO
 
 #define TUP_FMT         "(%s %s %s %s)"
-#define TUP_FMT_ELEM(e) ((e) ? sord_node_get_string(e) : "*")
+#define TUP_FMT_ELEM(e) ((e) ? sord_node_get_string(e) : (const uint8_t*)"*")
 #define TUP_FMT_ARGS(t) \
 	TUP_FMT_ELEM((t)[0]), \
 	TUP_FMT_ELEM((t)[1]), \
@@ -68,18 +68,18 @@
 
 /** Triple ordering */
 typedef enum {
-	SPO, ///<         Subject,   Predicate, Object
-	SOP, ///<         Subject,   Object,    Predicate
-	OPS, ///<         Object,    Predicate, Subject
-	OSP, ///<         Object,    Subject,   Predicate
-	PSO, ///<         Predicate, Subject,   Object
-	POS, ///<         Predicate, Object,    Subject
-	GSPO, ///< Graph,  Subject,   Predicate, Object
-	GSOP, ///< Graph,  Subject,   Object,    Predicate
-	GOPS, ///< Graph,  Object,    Predicate, Subject
-	GOSP, ///< Graph,  Object,    Subject,   Predicate
-	GPSO, ///< Graph,  Predicate, Subject,   Object
-	GPOS, ///< Graph,  Predicate, Object,    Subject
+	SPO,   ///<         Subject,   Predicate, Object
+	SOP,   ///<         Subject,   Object,    Predicate
+	OPS,   ///<         Object,    Predicate, Subject
+	OSP,   ///<         Object,    Subject,   Predicate
+	PSO,   ///<         Predicate, Subject,   Object
+	POS,   ///<         Predicate, Object,    Subject
+	GSPO,  ///< Graph,  Subject,   Predicate, Object
+	GSOP,  ///< Graph,  Subject,   Object,    Predicate
+	GOPS,  ///< Graph,  Object,    Predicate, Subject
+	GOSP,  ///< Graph,  Object,    Subject,   Predicate
+	GPSO,  ///< Graph,  Predicate, Subject,   Object
+	GPOS,  ///< Graph,  Predicate, Object,    Subject
 } SordOrder;
 
 /** String name of each ordering (array indexed by SordOrder) */
@@ -99,10 +99,10 @@ static const int orderings[NUM_ORDERS][TUP_LEN] = {
 
 /** World */
 struct SordWorldImpl {
-	GHashTable* names;    ///< URI or blank node identifier string => ID
-	GHashTable* langs;    ///< Language tag => Interned language tag
-	GHashTable* literals; ///< Literal => ID
-	size_t      n_nodes;  ///< Number of nodes
+	ZixHash* names;     ///< URI or blank node identifier string => ID
+	ZixHash* langs;     ///< Language tag => Interned language tag
+	ZixHash* literals;  ///< Literal => ID
+	size_t   n_nodes;   ///< Number of nodes
 };
 
 /** Store */
@@ -113,47 +113,48 @@ struct SordModelImpl {
 	 * If an index for e.g. SPO exists, it is a dictionary with
 	 * (S P O) keys (as a SordTuplrID), and ignored values.
 	 */
-	GSequence* indices[NUM_ORDERS];
+	ZixTree* indices[NUM_ORDERS];
 
 	size_t n_quads;
 };
 
 /** Mode for searching or iteration */
 typedef enum {
-	ALL,          ///< Iterate to end of store, returning all results, no filtering
-	SINGLE,       ///< Iteration over a single element (exact search)
-	RANGE,        ///< Iterate over range with equal prefix
-	FILTER_RANGE, ///< Iterate over range with equal prefix, filtering
-	FILTER_ALL    ///< Iterate to end of store, filtering
+	ALL,           ///< Iterate to end of store, returning all results, no filtering
+	SINGLE,        ///< Iteration over a single element (exact search)
+	RANGE,         ///< Iterate over range with equal prefix
+	FILTER_RANGE,  ///< Iterate over range with equal prefix, filtering
+	FILTER_ALL     ///< Iterate to end of store, filtering
 } SearchMode;
 
 /** Iterator over some range of a store */
 struct SordIterImpl {
-	const SordModel* sord;              ///< Store this is an iterator for
-	GSequenceIter*   cur;               ///< Current DB cursor
-	SordQuad         pat;               ///< Iteration pattern (in ordering order)
-	int              ordering[TUP_LEN]; ///< Store ordering
-	SearchMode       mode;              ///< Iteration mode
-	int              n_prefix;          ///< Length of range prefix (RANGE, FILTER_RANGE)
-	bool             end;               ///< True iff reached end
-	bool             skip_graphs;       ///< True iff iteration should ignore graphs
+	const SordModel* sord;               ///< Store this is an iterator for
+	ZixTreeIter*     cur;                ///< Current DB cursor
+	SordQuad         pat;                ///< Iteration pattern (in ordering order)
+	int              ordering[TUP_LEN];  ///< Store ordering
+	SearchMode       mode;               ///< Iteration mode
+	int              n_prefix;           ///< Length of range prefix (RANGE, FILTER_RANGE)
+	bool             end;                ///< True iff reached end
+	bool             skip_graphs;        ///< True iff iteration should ignore graphs
 };
 
 static unsigned
 sord_literal_hash(const void* n)
 {
 	SordNode* node = (SordNode*)n;
-	return g_str_hash(node->node.buf) + (node->lang ? g_str_hash(node->lang) : 0);
+	return zix_string_hash(node->node.buf)
+		+ (node->lang ? zix_string_hash(node->lang) : 0);
 }
 
-static gboolean
+static bool
 sord_literal_equal(const void* a, const void* b)
 {
 	SordNode* a_node = (SordNode*)a;
 	SordNode* b_node = (SordNode*)b;
 	return (a_node == b_node)
-		|| (g_str_equal(sord_node_get_string(a_node),
-		                sord_node_get_string(b_node))
+		|| (zix_string_equal(sord_node_get_string(a_node),
+		                     sord_node_get_string(b_node))
 		    && (a_node->lang == b_node->lang)
 		    && (a_node->datatype == b_node->datatype));
 }
@@ -162,32 +163,39 @@ SordWorld*
 sord_world_new(void)
 {
 	SordWorld* world = malloc(sizeof(struct SordWorldImpl));
-	world->names    = g_hash_table_new_full(g_str_hash, g_str_equal, 0, 0);
-	world->langs    = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, 0);
-	world->literals = g_hash_table_new_full(sord_literal_hash, sord_literal_equal, 0, 0);
+	world->names    = zix_hash_new(zix_string_hash, zix_string_equal);
+	world->langs    = zix_hash_new(zix_string_hash, zix_string_equal);
+	world->literals = zix_hash_new(sord_literal_hash, sord_literal_equal);
 	world->n_nodes  = 0;
 	return world;
 }
 
 static void
-free_hash_entry(void* key, void* value, void* user_data)
+free_node_entry(const void* key, void* value, void* user_data)
 {
 	SordNode* node = (SordNode*)value;
 	if (node->node.type == SERD_LITERAL) {
 		sord_node_free((SordWorld*)user_data, node->datatype);
 	}
-	g_free((uint8_t*)node->node.buf);
+	free((uint8_t*)node->node.buf);
 	free(node);
 }
 
+static void
+free_lang_entry(const void* key, void* value, void* user_data)
+{
+	free(value);
+}
+
 void
 sord_world_free(SordWorld* world)
 {
-	g_hash_table_foreach(world->literals, free_hash_entry, world);
-	g_hash_table_foreach(world->names, free_hash_entry, world);
-	g_hash_table_unref(world->names);
-	g_hash_table_unref(world->langs);
-	g_hash_table_unref(world->literals);
+	zix_hash_foreach(world->literals, free_node_entry, world);
+	zix_hash_foreach(world->names, free_node_entry, world);
+	zix_hash_foreach(world->langs, free_lang_entry, world);
+	zix_hash_free(world->names);
+	zix_hash_free(world->langs);
+	zix_hash_free(world->literals);
 	free(world);
 }
 
@@ -195,7 +203,7 @@ static inline int
 sord_node_compare(const SordNode* a, const SordNode* b)
 {
 	if (a == b || !a || !b) {
-		return (const char*)a - (const char*)b;
+		return 0;
 	} else if (a->node.type != b->node.type) {
 		return a->node.type - b->node.type;
 	}
@@ -220,7 +228,7 @@ sord_node_compare(const SordNode* a, const SordNode* b)
 		}
 		return cmp;
 	default:
-		break; // never reached
+		break;  // never reached
 	}
 	assert(false);
 	return 0;
@@ -274,8 +282,9 @@ sord_quad_compare(const void* x_ptr, const void* y_ptr, void* user_data)
 	for (int i = 0; i < TUP_LEN; ++i) {
 		const int idx = ordering[i];
 		const int cmp = sord_node_compare(x[idx], y[idx]);
-		if (cmp)
+		if (cmp) {
 			return cmp;
+		}
 	}
 
 	return 0;
@@ -285,18 +294,18 @@ static inline bool
 sord_iter_forward(SordIter* iter)
 {
 	if (!iter->skip_graphs) {
-		iter->cur = g_sequence_iter_next(iter->cur);
-		return g_sequence_iter_is_end(iter->cur);
+		iter->cur = zix_tree_iter_next(iter->cur);
+		return zix_tree_iter_is_end(iter->cur);
 	}
 
-	SordNode** key = (SordNode**)g_sequence_get(iter->cur);
+	SordNode** key = (SordNode**)zix_tree_get(iter->cur);
 	const SordQuad initial = { key[0], key[1], key[2], key[3] };
 	while (true) {
-		iter->cur = g_sequence_iter_next(iter->cur);
-		if (g_sequence_iter_is_end(iter->cur))
+		iter->cur = zix_tree_iter_next(iter->cur);
+		if (zix_tree_iter_is_end(iter->cur))
 			return true;
 
-		key = (SordNode**)g_sequence_get(iter->cur);
+		key = (SordNode**)zix_tree_get(iter->cur);
 		for (int i = 0; i < 3; ++i)
 			if (key[i] != initial[i])
 				return false;
@@ -312,9 +321,9 @@ static inline bool
 sord_iter_seek_match(SordIter* iter)
 {
 	for (iter->end = true;
-	     !g_sequence_iter_is_end(iter->cur);
+	     !zix_tree_iter_is_end(iter->cur);
 	     sord_iter_forward(iter)) {
-		const SordNode** const key = (const SordNode**)g_sequence_get(iter->cur);
+		const SordNode** const key = (const SordNode**)zix_tree_get(iter->cur);
 		if (sord_quad_match_inline(key, iter->pat))
 			return (iter->end = false);
 	}
@@ -333,25 +342,25 @@ sord_iter_seek_match_range(SordIter* iter)
 		return true;
 
 	do {
-		const SordNode** key = (const SordNode**)g_sequence_get(iter->cur);
+		const SordNode** key = (const SordNode**)zix_tree_get(iter->cur);
 
 		if (sord_quad_match_inline(key, iter->pat))
-			return false; // Found match
+			return false;  // Found match
 
 		for (int i = 0; i < iter->n_prefix; ++i) {
 			const int idx = iter->ordering[i];
 			if (!sord_id_match(key[idx], iter->pat[idx])) {
-				iter->end = true; // Reached end of valid range
+				iter->end = true;  // Reached end of valid range
 				return true;
 			}
 		}
 	} while (!sord_iter_forward(iter));
 
-	return (iter->end = true); // Reached end
+	return (iter->end = true);  // Reached end
 }
 
 static SordIter*
-sord_iter_new(const SordModel* sord, GSequenceIter* cur, const SordQuad pat,
+sord_iter_new(const SordModel* sord, ZixTreeIter* cur, const SordQuad pat,
               SordOrder order, SearchMode mode, int n_prefix)
 {
 	const int* ordering = orderings[order];
@@ -373,7 +382,7 @@ sord_iter_new(const SordModel* sord, GSequenceIter* cur, const SordQuad pat,
 	case SINGLE:
 	case RANGE:
 		assert(
-			sord_quad_match_inline((const SordNode**)g_sequence_get(iter->cur),
+			sord_quad_match_inline((const SordNode**)zix_tree_get(iter->cur),
 			                       iter->pat));
 		break;
 	case FILTER_RANGE:
@@ -387,7 +396,7 @@ sord_iter_new(const SordModel* sord, GSequenceIter* cur, const SordQuad pat,
 #ifdef SORD_DEBUG_ITER
 	SordQuad value;
 	sord_iter_get(iter, value);
-	SORD_ITER_LOG("New %p pat=" TUP_FMT " cur=" TUP_FMT " end=%d skipgraphs=%d\n",
+	SORD_ITER_LOG("New %p pat=" TUP_FMT " cur=" TUP_FMT " end=%d skip=%d\n",
 	              (void*)iter, TUP_FMT_ARGS(pat), TUP_FMT_ARGS(value),
 	              iter->end, iter->skip_graphs);
 #endif
@@ -403,7 +412,7 @@ sord_iter_get_model(SordIter* iter)
 void
 sord_iter_get(const SordIter* iter, SordQuad id)
 {
-	SordNode** key = (SordNode**)g_sequence_get(iter->cur);
+	SordNode** key = (SordNode**)zix_tree_get(iter->cur);
 	for (int i = 0; i < TUP_LEN; ++i) {
 		id[i] = key[i];
 	}
@@ -429,7 +438,7 @@ sord_iter_next(SordIter* iter)
 		case RANGE:
 			SORD_ITER_LOG("%p range next\n", (void*)iter);
 			// At the end if the MSNs no longer match
-			key = (const SordNode**)g_sequence_get(iter->cur);
+			key = (const SordNode**)zix_tree_get(iter->cur);
 			assert(key);
 			for (int i = 0; i < iter->n_prefix; ++i) {
 				const int idx = iter->ordering[i];
@@ -460,7 +469,8 @@ sord_iter_next(SordIter* iter)
 #ifdef SORD_DEBUG_ITER
 		SordQuad tup;
 		sord_iter_get(iter, tup);
-		SORD_ITER_LOG("%p Increment to " TUP_FMT "\n", (void*)iter, TUP_FMT_ARGS(tup));
+		SORD_ITER_LOG("%p Increment to " TUP_FMT "\n",
+		              (void*)iter, TUP_FMT_ARGS(tup));
 #endif
 		return false;
 	}
@@ -514,7 +524,7 @@ sord_best_index(SordModel* sord, const SordQuad pat, SearchMode* mode, int* n_pr
 		+ (pat[1] ? 1 : 0) * 0x010
 		+ (pat[2] ? 1 : 0) * 0x001;
 
-	SordOrder good[2];
+	SordOrder good[2] = { (SordOrder)-1, (SordOrder)-1 };
 
 	// Good orderings that don't require filtering
 	*mode     = RANGE;
@@ -530,10 +540,12 @@ sord_best_index(SordModel* sord, const SordQuad pat, SearchMode* mode, int* n_pr
 	case 0x111: *mode = SINGLE; return graph_search ? DEFAULT_GRAPH_ORDER : DEFAULT_ORDER;
 	}
 
-	if (sord_has_index(sord, &good[0], n_prefix, graph_search)) {
-		return good[0];
-	} else if (sord_has_index(sord, &good[1], n_prefix, graph_search)) {
-		return good[1];
+	if (*mode == RANGE) {
+		if (sord_has_index(sord, &good[0], n_prefix, graph_search)) {
+			return good[0];
+		} else if (sord_has_index(sord, &good[1], n_prefix, graph_search)) {
+			return good[1];
+		}
 	}
 
 	// Not so good orderings that require filtering, but can
@@ -571,10 +583,15 @@ sord_new(SordWorld* world, unsigned indices, bool graphs)
 	sord->n_quads = 0;
 
 	for (unsigned i = 0; i < (NUM_ORDERS / 2); ++i) {
+		const int* const ordering   = orderings[i];
+		const int* const g_ordering = orderings[i + (NUM_ORDERS / 2)];
+
 		if (indices & (1 << i)) {
-			sord->indices[i] = g_sequence_new(NULL);
+			sord->indices[i] = zix_tree_new(
+				false, sord_quad_compare, (void*)ordering);
 			if (graphs) {
-				sord->indices[i + (NUM_ORDERS / 2)] = g_sequence_new(NULL);
+				sord->indices[i + (NUM_ORDERS / 2)] = zix_tree_new(
+					false, sord_quad_compare, (void*)g_ordering);
 			} else {
 				sord->indices[i + (NUM_ORDERS / 2)] = NULL;
 			}
@@ -585,7 +602,8 @@ sord_new(SordWorld* world, unsigned indices, bool graphs)
 	}
 
 	if (!sord->indices[DEFAULT_ORDER]) {
-		sord->indices[DEFAULT_ORDER] = g_sequence_new(NULL);
+		sord->indices[DEFAULT_ORDER] = zix_tree_new(
+			false, sord_quad_compare, (void*)orderings[DEFAULT_ORDER]);
 	}
 
 	return sord;
@@ -596,18 +614,18 @@ sord_node_free_internal(SordWorld* world, SordNode* node)
 {
 	assert(node->refs == 0);
 	if (node->node.type == SERD_LITERAL) {
-		if (!g_hash_table_remove(world->literals, node)) {
+		if (zix_hash_remove(world->literals, node)) {
 			fprintf(stderr, "Failed to remove literal from hash.\n");
 			return;
 		}
 		sord_node_free(world, node->datatype);
 	} else {
-		if (!g_hash_table_remove(world->names, node->node.buf)) {
+		if (zix_hash_remove(world->names, node->node.buf)) {
 			fprintf(stderr, "Failed to remove resource from hash.\n");
 			return;
 		}
 	}
-	g_free((uint8_t*)node->node.buf);
+	free((uint8_t*)node->node.buf);
 	free(node);
 }
 
@@ -658,16 +676,16 @@ sord_free(SordModel* sord)
 	sord_iter_free(i);
 
 	// Free quads
-	for (GSequenceIter* i = g_sequence_get_begin_iter(sord->indices[DEFAULT_ORDER]);
-	     !g_sequence_iter_is_end(i);
-	     i = g_sequence_iter_next(i)) {
-		free(g_sequence_get(i));
+	for (ZixTreeIter* i = zix_tree_begin(sord->indices[DEFAULT_ORDER]);
+	     !zix_tree_iter_is_end(i);
+	     i = zix_tree_iter_next(i)) {
+		free(zix_tree_get(i));
 	}
 
 	// Free indices
 	for (unsigned i = 0; i < NUM_ORDERS; ++i)
 		if (sord->indices[i])
-			g_sequence_free(sord->indices[i]);
+			zix_tree_free(sord->indices[i]);
 
 	free(sord);
 }
@@ -696,67 +714,44 @@ sord_begin(const SordModel* sord)
 	if (sord_num_quads(sord) == 0) {
 		return NULL;
 	} else {
-		GSequenceIter* cur = g_sequence_get_begin_iter(sord->indices[DEFAULT_ORDER]);
+		ZixTreeIter* cur = zix_tree_begin(sord->indices[DEFAULT_ORDER]);
 		SordQuad pat = { 0, 0, 0, 0 };
 		return sord_iter_new(sord, cur, pat, DEFAULT_ORDER, ALL, 0);
 	}
 }
 
-static inline GSequenceIter*
-index_search(GSequence* db, const SordQuad search_key, const int* ordering)
+static inline ZixTreeIter*
+index_search(ZixTree* db, const SordQuad search_key)
 {
-	return g_sequence_search(
-		db, (void*)search_key, sord_quad_compare, (void*)ordering);
+	ZixTreeIter* iter = NULL;
+	zix_tree_find(db, (void*)search_key, &iter);
+	return iter;
 }
 
-static inline GSequenceIter*
-index_lower_bound_iter(GSequenceIter* i, const SordQuad search_key)
+static inline ZixTreeIter*
+index_lower_bound(ZixTree* db, const SordQuad search_key)
 {
-	/* i is now at the position where search_key would be inserted,
-	   but this is not necessarily a match, and we need the leftmost...
-	*/
-
-	if (g_sequence_iter_is_begin(i)) {
-		return i;
-	} else if (g_sequence_iter_is_end(i)) {
-		i = g_sequence_iter_prev(i);
+	ZixTreeIter* iter = NULL;
+	zix_tree_find(db, (void*)search_key, &iter);
+	if (!iter) {
+		return NULL;
 	}
 
-	if (!sord_quad_match_inline(search_key, g_sequence_get(i))) {
-		// No match, but perhaps immediately after a match
-		GSequenceIter* const prev = g_sequence_iter_prev(i);
-		if (!sord_quad_match_inline(search_key, g_sequence_get(prev))) {
-			return i; // No match (caller must check)
-		} else {
-			i = prev;
+	ZixTreeIter* prev = NULL;
+	while ((prev = zix_tree_iter_prev(iter))) {
+		if (!prev) {
+			return iter;
 		}
-	}
 
-	/* i now points to some matching element,
-	   but not necessarily the first...
-	*/
-	assert(sord_quad_match_inline(search_key, g_sequence_get(i)));
-
-	while (!g_sequence_iter_is_begin(i)) {
-		if (sord_quad_match_inline(search_key, g_sequence_get(i))) {
-			GSequenceIter* const prev = g_sequence_iter_prev(i);
-			if (sord_quad_match_inline(search_key, g_sequence_get(prev))) {
-				i = prev;
-				continue;
-			}
+		const SordNode** const key = (const SordNode**)zix_tree_get(prev);
+		if (!sord_quad_match_inline(key, search_key)) {
+			return iter;
 		}
-		break;
+
+		iter = prev;
 	}
 
-	return i;
-}
-
-static inline GSequenceIter*
-index_lower_bound(GSequence* db, const SordQuad search_key, const int* ordering)
-{
-	GSequenceIter* i = g_sequence_search(
-		db, (void*)search_key, sord_quad_compare, (void*)ordering);
-	return index_lower_bound_iter(i, search_key);
+	return iter;
 }
 
 SordIter*
@@ -765,25 +760,24 @@ sord_find(SordModel* sord, const SordQuad pat)
 	if (!pat[0] && !pat[1] && !pat[2] && !pat[3])
 		return sord_begin(sord);
 
-	SearchMode          mode;
-	int                 prefix_len;
-	const SordOrder     index_order = sord_best_index(sord, pat, &mode, &prefix_len);
-	const int* const    ordering    = orderings[index_order];
+	SearchMode      mode;
+	int             prefix_len;
+	const SordOrder index_order = sord_best_index(sord, pat, &mode, &prefix_len);
 
 	SORD_FIND_LOG("Find " TUP_FMT "  index=%s  mode=%d  prefix_len=%d ordering=%d%d%d%d\n",
 	              TUP_FMT_ARGS(pat), order_names[index_order], mode, prefix_len,
 	              ordering[0], ordering[1], ordering[2], ordering[3]);
 
 	if (pat[0] && pat[1] && pat[2] && pat[3])
-		mode = SINGLE; // No duplicate quads (Sord is a set)
+		mode = SINGLE;  // No duplicate quads (Sord is a set)
 
-	GSequence* const     db  = sord->indices[index_order];
-	GSequenceIter* const cur = index_lower_bound(db, pat, ordering);
-	if (g_sequence_iter_is_end(cur)) {
+	ZixTree* const     db  = sord->indices[index_order];
+	ZixTreeIter* const cur = index_lower_bound(db, pat);
+	if (zix_tree_iter_is_end(cur)) {
 		SORD_FIND_LOG("No match found\n");
 		return NULL;
 	}
-	const SordNode** const key = (const SordNode**)g_sequence_get(cur);
+	const SordNode** const key = (const SordNode**)zix_tree_get(cur);
 	if (!key || ( (mode == RANGE || mode == SINGLE)
 	              && !sord_quad_match_inline(pat, key) )) {
 		SORD_FIND_LOG("No match found\n");
@@ -793,10 +787,27 @@ sord_find(SordModel* sord, const SordQuad pat)
 	return sord_iter_new(sord, cur, pat, index_order, mode, prefix_len);
 }
 
+bool
+sord_contains(SordModel* sord, const SordQuad pat)
+{
+	SordIter* iter = sord_find(sord, pat);
+	bool      ret  = (iter != NULL);
+	sord_iter_free(iter);
+	return ret;
+}
+
 static SordNode*
 sord_lookup_name(SordWorld* world, const uint8_t* str)
 {
-	return g_hash_table_lookup(world->names, str);
+	return zix_hash_find(world->names, str);
+}
+
+char*
+sord_strndup(const char* str, size_t len)
+{
+	char* dup = malloc(len + 1);
+	memcpy(dup, str, len + 1);
+	return dup;
 }
 
 static SordNode*
@@ -809,7 +820,7 @@ sord_new_node(SerdType type, const uint8_t* data,
 	node->datatype     = datatype;
 	node->refs         = 1;
 	node->refs_as_obj  = 0;
-	node->node.buf     = (uint8_t*)g_strdup((const char*)data);
+	node->node.buf     = (uint8_t*)sord_strndup((const char*)data, n_bytes);
 	node->node.n_bytes = n_bytes;
 	node->node.n_chars = n_chars;
 	node->node.flags   = flags;
@@ -822,10 +833,10 @@ const char*
 sord_intern_lang(SordWorld* world, const char* lang)
 {
 	if (lang) {
-		char* ilang = g_hash_table_lookup(world->langs, lang);
+		char* ilang = zix_hash_find(world->langs, lang);
 		if (!ilang) {
-			ilang = g_strdup(lang);
-			g_hash_table_insert(world->langs, ilang, ilang);
+			ilang = sord_strndup(lang, strlen(lang));
+			zix_hash_insert(world->langs, ilang, ilang);
 		}
 		lang = ilang;
 	}
@@ -848,12 +859,7 @@ sord_lookup_literal(SordWorld* world, SordNode* type,
 	key.node.flags   = 0;
 	key.node.type    = SERD_LITERAL;
 
-	SordNode* id = g_hash_table_lookup(world->literals, &key);
-	if (id) {
-		return id;
-	} else {
-		return 0;
-	}
+	return zix_hash_find(world->literals, &key);
 }
 
 SordNodeType
@@ -926,8 +932,8 @@ sord_new_uri_counted(SordWorld* world, const uint8_t* str,
 	}
 
 	node = sord_new_node(SERD_URI, str, n_bytes, n_chars, 0, 0, 0);
-	assert(!g_hash_table_lookup(world->names, node->node.buf));
-	g_hash_table_insert(world->names, (char*)node->node.buf, node);
+	assert(!zix_hash_find(world->names, node->node.buf));
+	zix_hash_insert(world->names, (char*)node->node.buf, node);
 	sord_add_node(world, node);
 	return node;
 }
@@ -950,7 +956,7 @@ sord_new_blank_counted(SordWorld* world, const uint8_t* str,
 	}
 
 	node = sord_new_node(SERD_BLANK, str, n_bytes, n_chars, 0, 0, 0);
-	g_hash_table_insert(world->names, (char*)node->node.buf, node);
+	zix_hash_insert(world->names, (char*)node->node.buf, node);
 	sord_add_node(world, node);
 	return node;
 }
@@ -978,7 +984,7 @@ sord_new_literal_counted(SordWorld* world, SordNode* datatype,
 	node = sord_new_node(SERD_LITERAL,
 	                     str, n_bytes, n_chars, flags,
 	                     sord_node_copy(datatype), lang);
-	g_hash_table_insert(world->literals, node, node);  // FIXME: correct?
+	zix_hash_insert(world->literals, node, node);  // FIXME: correct?
 	sord_add_node(world, node);
 	assert(node->refs == 1);
 	return node;
@@ -1051,7 +1057,7 @@ sord_node_from_serd_node(SordWorld*      world,
 		buf[uri_len] = '\0';
 		SordNode* ret = sord_new_uri_counted(
 			world, buf, uri_prefix.len + uri_suffix.len,
-			uri_prefix.len + uri_suffix.len); // FIXME: UTF-8
+			uri_prefix.len + uri_suffix.len);  // FIXME: UTF-8
 		return ret;
 	}
 	case SERD_BLANK:
@@ -1092,17 +1098,7 @@ sord_node_copy(const SordNode* node)
 static inline bool
 sord_add_to_index(SordModel* sord, const SordNode** tup, SordOrder order)
 {
-	assert(sord->indices[order]);
-	const int* const     ordering = orderings[order];
-	GSequenceIter* const cur      = index_search(sord->indices[order], tup, ordering);
-	GSequenceIter* const match    = index_lower_bound_iter(cur, tup);
-	if (!g_sequence_iter_is_end(match)
-	    && !sord_quad_compare(g_sequence_get(match), tup, (void*)ordering)) {
-		return false;  // Quad already stored in this index
-	}
-
-	g_sequence_insert_before(cur, tup);
-	return true;
+	return !zix_tree_insert(sord->indices[order], tup, NULL);
 }
 
 bool
@@ -1120,9 +1116,9 @@ sord_add(SordModel* sord, const SordQuad tup)
 	for (unsigned i = 0; i < NUM_ORDERS; ++i) {
 		if (sord->indices[i]) {
 			if (!sord_add_to_index(sord, quad, i)) {
-				assert(i == 0); // Assuming index coherency
+				assert(i == 0);  // Assuming index coherency
 				free(quad);
-				return false; // Quad already stored, do nothing
+				return false;  // Quad already stored, do nothing
 			}
 		}
 	}
@@ -1131,7 +1127,7 @@ sord_add(SordModel* sord, const SordQuad tup)
 		sord_add_quad_ref(sord, tup[i], i);
 
 	++sord->n_quads;
-	assert(sord->n_quads == (size_t)g_sequence_get_length(sord->indices[SPO]));
+	//assert(sord->n_quads == (size_t)zix_tree_get_length(sord->indices[SPO]));
 	return true;
 }
 
@@ -1143,17 +1139,15 @@ sord_remove(SordModel* sord, const SordQuad tup)
 	SordNode** quad = NULL;
 	for (unsigned i = 0; i < NUM_ORDERS; ++i) {
 		if (sord->indices[i]) {
-			const int* const     ordering = orderings[i];
-			GSequenceIter* const cur      = index_lower_bound(
-				sord->indices[i], tup, ordering);
-			if (!g_sequence_iter_is_end(cur)) {
+			ZixTreeIter* const cur = index_search(sord->indices[i], tup);
+			if (!zix_tree_iter_is_end(cur)) {
 				if (!quad) {
-					quad = g_sequence_get(cur);
+					quad = zix_tree_get(cur);
 				}
-				g_sequence_remove(cur);
+				zix_tree_remove(sord->indices[i], cur);
 			} else {
-				assert(i == 0); // Assuming index coherency
-				return; // Quad not found, do nothing
+				assert(i == 0);  // Assuming index coherency
+				return;  // Quad not found, do nothing
 			}
 		}
 	}
